#include "nsISupports.h"
#include "xmmsConnection.h"
#include "xmmsclient-moz.h"
#include "xmmsUtils.h"

NS_IMPL_ISUPPORTS1(xmmsConnection, xmmsIConnection)

xmmsConnection::xmmsConnection(const std::string& name)
 : mClient(name),
   mMedialib(mClient.medialib),
   mCollections(mClient.collection),
   mPlaylists(mClient.playlist),
   mPlayback(mClient.playback),
   mSocketHandler(0)
{
}

xmmsConnection::~xmmsConnection()
{
    teardown();
}

void xmmsConnection::teardown()
{
    delete mSocketHandler;
    mSocketHandler = 0;
}

/* short connect (in string path); */
NS_IMETHODIMP xmmsConnection::Connect(const char *path)
{
    XMMS_TRY
    NS_ENSURE_FALSE(mSocketHandler, NS_ERROR_ALREADY_INITIALIZED);
    NS_ENSURE_FALSE(mClient.isConnected(), NS_ERROR_ALREADY_INITIALIZED);

    // strangely, I have to connect *before* setting the mainloop, otherwise setMainloop() fails with "not connected"
    mClient.connect(path);
    std::cout << "connected" << std::endl;

    xmmsc_connection_t* xmmsConn = (xmmsc_connection_t*) mClient.getConnection(); // HACK need non-const
    mSocketHandler = new xmmsSocketHandler(xmmsConn);
    mSocketHandler->Init();
    //mClient.setMainloop(mSocketHandler);
    std::cout << "mainloop set done" << std::endl;

    return NS_OK;
    XMMS_CATCH
}

/* void disconnect (); */
NS_IMETHODIMP xmmsConnection::Disconnect()
{
    XMMS_TRY
    NS_ENSURE_FALSE(mSocketHandler, NS_ERROR_ALREADY_INITIALIZED);
    NS_ENSURE_FALSE(mClient.isConnected(), NS_ERROR_ALREADY_INITIALIZED);

    xmmsc_io_disconnect(mClient.getConnection());
    teardown();

    return NS_OK;
    XMMS_CATCH
}

/* readonly attribute boolean connected; */
NS_IMETHODIMP xmmsConnection::GetConnected(PRBool* ret)
{
    NS_ENSURE_ARG_POINTER(ret);
    *ret = mClient.isConnected();
    return NS_OK;
}

/* void addConnectedObserver (in xmmsIObserver o); */
NS_IMETHODIMP xmmsConnection::AddConnectedObserver(xmmsIObserver *o)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void removeConnectedObserver (in xmmsIObserver o); */
NS_IMETHODIMP xmmsConnection::RemoveConnectedObserver(xmmsIObserver *o)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void quitServer (); */
NS_IMETHODIMP xmmsConnection::QuitServer()
{
    mClient.quit();
    return NS_OK;
}

/* readonly attribute xmmsIMedialib medialib; */
NS_IMETHODIMP xmmsConnection::GetMedialib(xmmsIMedialib** ret)
{
    NS_ENSURE_ARG_POINTER(ret);
    *ret = &mMedialib;
    NS_ADDREF(*ret);
    return NS_OK;
}

/* readonly attribute xmmsICollections collections; */
NS_IMETHODIMP xmmsConnection::GetCollections(xmmsICollections** ret)
{
    NS_ENSURE_ARG_POINTER(ret);
    *ret = &mCollections;
    NS_ADDREF(*ret);
    return NS_OK;
}

/* readonly attribute xmmsIPlaylists playlists; */
NS_IMETHODIMP xmmsConnection::GetPlaylists(xmmsIPlaylists** ret)
{
    NS_ENSURE_ARG_POINTER(ret);
    *ret = &mPlaylists;
    NS_ADDREF(*ret);
    return NS_OK;
}

/* readonly attribute xmmsIPlayback playback; */
NS_IMETHODIMP xmmsConnection::GetPlayback(xmmsIPlayback** ret)
{
    NS_ENSURE_ARG_POINTER(ret);
    *ret = &mPlayback;
    NS_ADDREF(*ret);
    return NS_OK;
}
