 /**
  * XMMS client lib mainloop / poll() implementation for Mozilla.
  *
  * xmms client lib does its own I/O (networking), via a FD, and so needs to be notified by Mozilla
  * when there's new data in the incoming queue, and get notified when the outgoing queue is ready to
  * accept (more) data.
  *
  * Mozilla does the poll() in nsSocketTransportService2.cpp, and offers an acceptSocket(),
  * which takes an nsASocketHandler. We implement the latter.
  * nsSocketTransportService expects an NSPR PRFileDesc from the nsASocketHandler,
  * which unfortunately is not the same as the OS FD, and NSPR's convert method is private.
  */

#ifndef __xmmsclient_moz_h__
#define __xmmsclient_moz_h__

//extern "C" {
#include "xmmsclient/xmmsclient.h"
//}
#include <xmmsclient/xmmsclient++/mainloop.h>
#include "private/pprio.h"
#include "nsISupports.h"
#include "nsASocketHandler.h"

class xmmsSocketHandler : public nsASocketHandler, public Xmms::MainloopInterface
{
    NS_DECL_ISUPPORTS

    xmmsSocketHandler(xmmsc_connection_t *c);
    void Init();

    virtual void run() {}

    virtual void OnSocketReady(PRFileDesc *fd, PRInt16 outFlags);

    // called when a socket is no longer under the control of the socket
    // transport service.  the socket handler may close the socket at this
    // point. after this call returns, the handler will no longer be owned
    // by the socket transport service.
    virtual void OnSocketDetached(PRFileDesc *fd);

protected:
    xmmsc_connection_t* mXMMSConn;
};

#endif
